#! /usr/bin/env python3
"""----------------------------------------------------------------------------
  ==  == ====== ====   ====    |
                    \\     ||  | Unsupported Contributions for OpenFOAM
  ======   //   ||  || ===//   |
  ||  ||  //    ||  // || \\   | Copyright (C) 2020-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Script
    pipeline_bot.py

Author
    F. Schlegel

Description
    Bot to label merge requests and issues in a nightly build pipeline

----------------------------------------------------------------------------"""

import datetime
import argparse
import gitlab

def isold(updated_at):
    "Check if latest update of merge request or issue is older than a month"
    date = updated_at.split('T')[0].split('-')
    year = int(date[0])
    month = int(date[1])
    day = int(date[2])
    delta = datetime.date.today() - datetime.date(year, month, day)
    return delta.days > 30

parser = argparse.ArgumentParser(description='Checks merge requests and issues \
and adds labels to them if required.')

# Add positional arguments
parser.add_argument('--id', required=True, help='project id')
parser.add_argument('--token', required=True, help='private token')
parser.add_argument('--url', required=True, help='gitlab url')

args = parser.parse_args()

# Connect to gitlab project
globject = gitlab.Gitlab(args.url, private_token=args.token)
project = globject.projects.get(args.id)

# Loop through merge requests
mrs = project.mergerequests.list(state='opened')

for mr in mrs:
    # Ignore draft or work in progress merge requests
    if mr.title.startswith('WIP:') or mr.title.startswith('Draft:'):
        print('Skipping draft merge request:', mr.iid)
    else:
        # Notify if no assignee is set
        if not mr.assignees:
            print('Merge request has no assignee:', mr.iid)
            mr.labels.append('Information: Assignee required')
            mr.save()
        # Warn if author is also assignee
        elif mr.assignees[0]['id'] == mr.author['id']:
            print('Merge request has self-assignment:', mr.iid)
            mr.labels.append('Warning: Self-assignment')
            mr.save()
        # Notify is no reviewer is set
        if not mr.reviewers:
            print('Merge request has no reviewers:', mr.iid)
            mr.labels.append('Information: Reviewer required')
            mr.save()
        # Warn if no milestone is set
        if not mr.milestone:
            print('Merge request has no milestone:', mr.iid)
            mr.labels.append('Warning: No Milestone selected')
            mr.save()
        # Warn if merge request is outdated
        if isold(mr.updated_at):
            print('Merge request is older than 1 month:', mr.iid)
            mr.labels.append('Warning: Stale merge request')
            mr.save()
        else:
            try:
                mr.labels.remove('Warning: Stale merge request')
                mr.save()
            except ValueError:
                pass

#Loop through all open issues
issues = project.issues.list(state='opened')

for issue in issues:
    # Warn if no milestone is set
    if not issue.milestone:
        print('Issue has no milestone:', issue.iid)
        issue.labels.append('Warning: No Milestone selected')
        issue.save()
    # Warn if issue is outdated
    if isold(issue.updated_at):
        print('Issue is older than 1 month:', issue.iid)
        issue.labels.append('Warning: Stale issue')
        issue.save()
    else:
        try:
            issue.labels.remove('Warning: Stale issue')
            issue.save()
        except ValueError:
            pass

#Loop through all issues and check scoped labels
scoped_labels = ['Foundation: Considered for Contribution',
                 'Foundation: Solved in HZDR Version',
                 'Foundation: Submitted to Foundation',
                 'Foundation: Requires Backport / Sprint Cycle',
                 'Foundation: Contributed']

for i in range(len(scoped_labels)-1):
    issues = project.issues.list(labels=scoped_labels[i])

    for issue in issues:
        if scoped_labels[i+1] in issue.labels:
            print('Removed label', scoped_labels[i], 'from issue:', issue.iid)
            issue.labels.remove(scoped_labels[i])
            issue.save()
        elif issue.state == 'closed':
            print('Issue was reopened:', issue.iid)
            issue.labels.append('Information: Issue reopened automatically')
            issue.state_event = 'reopen'
            issue.save()

issues = project.issues.list(labels=scoped_labels[-1])
for issue in issues:
    if issue.state == 'opened':
        print('Issue was closed:', issue.iid)
        issue.labels.append('Information: Issue closed automatically')
        issue.state_event = 'close'
        issue.save()
